from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Task


class Test(TestCase):

    def test_task(self):
        task = Task('uuid', 'name', 'product_uuid', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        task_string = json_encode(task)
        expected_string = '{"_type": "task", "uuid": "uuid", "name": "name", "product_uuid": "product_uuid", ' \
                          '"pose": {"x": 0.0, "y": 0.0, "z": 0.0, "rx": 0.0, "ry": 0.0, "rz": 0.0}, ' \
                          '"automated": false, "conditions": [], "description": ""}'

        assert type(task_string) == str
        assert task_string == expected_string, f"Expected {expected_string}, got: {task_string}"

        task_decoded = json_decode(task_string)
        assert type(task_decoded) == Task, f"Got: {type(task_decoded)}. Expected {Task}"
        assert task_decoded == task, "The decoded object must be equal to the encoded"

    def test_task_equals(self):
        task1 = Task('uuid', 'name', 'product_uuid', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        task2 = Task('uuid', 'name', 'product_uuid', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})
        task3 = Task('other', 'name', 'product_uuid', {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0})

        assert task1 == task2
        assert task1 != task3
        assert task1 != "not status"

    def test_task_repr(self):
        task = Task('uuid', 'name', 'product_uuid',
                         {'x': 0.0, 'y': 0.0, 'z': 0.0, 'rx': 0.0, 'ry': 0.0, 'rz': 0.0}, description="Hard task!")
        expected, got = 'Hard task!', f'{task}'

        assert expected in got, f"Expected {expected} in got: {got}"




from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import AddProcess, MoveTask, CompleteTask, Destination, ConfirmNeeded


class TestAddProcess(TestCase):

    def test_add_process(self):
        add_process = AddProcess('uuid')
        add_process_string = json_encode(add_process)
        expected_string = '{"_type": "add_process", "process_uuid": "uuid"}'

        assert type(add_process_string) == str
        assert add_process_string == expected_string, f"Expected {expected_string}, got: {add_process_string}"

        add_process_decoded = json_decode(add_process_string)
        assert type(add_process_decoded) == AddProcess, f"Got: {type(add_process_decoded)}. Expected {AddProcess}"
        assert add_process_decoded == add_process, "The decoded object must be equal to the encoded"

    def test_add_process_equals(self):
        at1 = AddProcess('uuid')
        at2 = AddProcess('uuid')
        at3 = AddProcess('uuid2')

        assert at1 == at2
        assert at1 != at3
        assert at1 != "not status"

    def test_add_process_repr(self):
        add_task = AddProcess('uuid')
        expected, got = 'uuid', f'{add_task}'

        assert expected in got, f"Expected {expected} in got: {got}"


class TestMoveTask(TestCase):

    def test_move_task(self):
        move_task = MoveTask('uuid', Destination.Robot)
        move_task_string = json_encode(move_task)
        expected_string = '{"_type": "move_task", "task_uuid": "uuid", "destination": "Robot"}'

        assert type(move_task_string) == str
        assert move_task_string == expected_string, f"Expected {expected_string}, got: {move_task_string}"

        move_task_decoded = json_decode(move_task_string)
        assert type(move_task_decoded) == MoveTask, f"Got: {type(move_task_decoded)}. " \
                                                              f"Expected {MoveTask}"
        assert move_task_decoded == move_task, f"The decoded object: {move_task_decoded} " \
                                                         f"must be equal to the encoded: {move_task}"

    def test_move_task_equals(self):
        mo1 = MoveTask('uuid', Destination.Human)
        mo2 = MoveTask('uuid', Destination.Human)
        mo3 = MoveTask('uuid', Destination.Robot)

        assert mo1 == mo2, f"This: {mo1}, should be equal to this: {mo2}"
        assert mo1 != mo3, f"This: {mo1}, should not be equal to this: {mo3}"
        assert mo1 != "not status"

    def test_move_task_repr(self):
        move_task = MoveTask('uuid', Destination.Robot)
        expected, got = 'Robot', f'{move_task}'

        assert expected in got, f"Expected {expected} in got: {got}"


class TestCompleteTask(TestCase):

    def test_complete_task(self):
        complete_task = CompleteTask('uuid')
        complete_task_string = json_encode(complete_task)
        expected_string = '{"_type": "complete_task", "task_uuid": "uuid"}'

        assert type(complete_task_string) == str
        assert complete_task_string == expected_string, f"Expected {expected_string}, got: {complete_task_string}"

        complete_task_decoded = json_decode(complete_task_string)
        assert type(complete_task_decoded) == CompleteTask, f"Got: {type(complete_task_decoded)}. " \
                                                                      f"Expected {CompleteTask}"
        assert complete_task_decoded == complete_task, f"The decoded object: {complete_task_decoded} " \
                                                         f"must be equal to the encoded: {complete_task}"

    def test_complete_task_equals(self):
        co1 = CompleteTask('uuid')
        co2 = CompleteTask('uuid')
        co3 = CompleteTask('uuid2')

        assert co1 == co2
        assert co1 != co3
        assert co1 != "not status"

    def test_complete_task_repr(self):
        complete_task = CompleteTask('uuid')
        expected, got = 'uuid', f'{complete_task}'

        assert expected in got, f"Expected {expected} in got: {got}"


class TestConfirmNeeded(TestCase):

    def test_confirm_needed(self):
        confirm_needed = ConfirmNeeded('uuid', True)
        confirm_needed_string = json_encode(confirm_needed)
        expected_string = '{"_type": "confirm_needed", "task_uuid": "uuid", "confirm_needed": true}'

        assert type(confirm_needed_string) == str
        assert confirm_needed_string == expected_string, f"Expected {expected_string}, got: {confirm_needed_string}"

        confirm_needed_decoded = json_decode(confirm_needed_string)
        assert type(confirm_needed_decoded) == ConfirmNeeded, f"Got: {type(confirm_needed_decoded)}. " \
                                                              f"Expected {ConfirmNeeded}"
        assert confirm_needed_decoded == confirm_needed, f"The decoded object: {confirm_needed_decoded} " \
                                                         f"must be equal to the encoded: {confirm_needed}"

    def test_confirm_needed_equals(self):
        co1 = ConfirmNeeded('uuid', True)
        co2 = ConfirmNeeded('uuid',  True)
        co3 = ConfirmNeeded('uuid', False)

        assert co1 == co2
        assert co1 != co3
        assert co1 != "not status"

    def test_confirm_needed_repr(self):
        confirm_needed = ConfirmNeeded('uuid', True)
        expected, got = 'uuid', f'{confirm_needed}'

        assert expected in got, f"Expected {expected} in got: {got}"



from unittest import TestCase

from holour import json_encode, json_decode
from holour.msg import Process


class Test(TestCase):

    def test_process(self):
        process = Process('uuid_1', 'build house', ['task_1', 'task_2'], "Build a beautiful house.")
        process_string = json_encode(process)
        expected_string = '{"_type": "process", "uuid": "uuid_1", "name": "build house", ' \
                          '"tasks": ["task_1", "task_2"], "description": "Build a beautiful house."}'

        self.assertIsInstance(process_string, str)
        self.assertEqual(process_string, expected_string, f"Expected {expected_string}, got: {process_string}")

        process_decoded = json_decode(process_string)
        assert type(process_decoded) == Process, f"Got: {type(process_decoded)}. Expected {Process}"
        assert process_decoded == process, "The decoded object must be equal to the encoded"

    def test_task_equals(self):
        t1 = Process('uuid_1', 'build house', ['task_1', 'task_1'], "Build a beautiful house.")
        t2 = Process('uuid_1', 'build house', ['task_1', 'task_1'], "Build a beautiful house.")
        t3 = Process('uuid_2', 'build house', ['task_1', 'task_1'])

        assert t1 == t2
        assert t1 != t3
        assert t1 != "not status"

    def test_task_repr(self):
        process = Process('uuid_1', 'build house', ['task_1', 'task_1'], "Build a beautiful house.")
        expected, got = 'Build a beautiful house.', f'{process}'

        assert expected in got, f"Expected {expected} in got: {got}"

    def test_sub_tasks_or_operations(self):
        with self.assertRaises(ValueError):
            Process('uuid_1', 'task_1', [])



from typing import List


class Task:

    def __init__(self, uuid: str, name: str, product_uuid: str, pose: dict, automated: bool = False,
                 conditions: List[str] = None, description: str = "", _type: str = ''):
        conditions = conditions if conditions else []
        assert type(uuid) == str
        assert type(name) == str
        assert type(product_uuid) == str
        assert type(pose) == dict
        assert type(automated) == bool
        assert type(conditions) == list, f"Type of conditions: {type(conditions)}"
        assert type(description) == str
        for key in ['x', 'y', 'z', 'rx', 'ry', 'rz']:
            assert key in pose, f"The following must be in pose-dict: {key}. Pose: {pose}"
            assert type(pose[key]) == float, f"{key} must be of type float instead of: {type(pose[key])}."

        self._type = 'task'
        self.uuid = uuid
        self.name = name
        self.product_uuid = product_uuid
        self.pose = pose
        self.automated = automated
        self.conditions = conditions
        self.description = description

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Task):
            return other.uuid == self.uuid \
                   and other.product_uuid == self.product_uuid \
                   and other.pose == self.pose \
                   and other.automated == self.automated \
                   and other.conditions == self.conditions \
                   and other.description == self.description
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<uuid={self.uuid},name={self.name},product_uuid={self.product_uuid},pose={self.pose}," \
               f"automated={self.automated},conditions={self.conditions},description={self.description}>"



from enum import Enum
from typing import Union


class AddProcess:

    def __init__(self, process_uuid: str, _type: str = ''):
        self._type = 'add_process'
        self.process_uuid = process_uuid

    def __eq__(self, other: object) -> bool:
        if isinstance(other, AddProcess):
            return other.process_uuid == self.process_uuid
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<process_uuid={self.process_uuid}>"


class Destination(Enum):
    Human = 'Human'
    Robot = 'Robot'

    @staticmethod
    def from_str(label: str):
        if label in ('Human', 'human'):
            return Destination.Human
        elif label in ('Robot', 'robot'):
            return Destination.Robot
        else:
            raise NotImplementedError


class MoveTask:

    def __init__(self, task_uuid: str, destination: Union[str, Destination], _type: str = ''):
        self._type = 'move_task'
        self.task_uuid = task_uuid
        self.destination = destination if type(destination) == Destination else Destination.from_str(destination)

    def __eq__(self, other: object) -> bool:
        if isinstance(other, MoveTask):
            return other.task_uuid == self.task_uuid and \
                   other.destination == self.destination
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<task_uuid={self.task_uuid},destination={self.destination}>"


class CompleteTask:

    def __init__(self, task_uuid: str, _type: str = ''):
        self._type = 'complete_task'
        self.task_uuid = task_uuid

    def __eq__(self, other: object) -> bool:
        if isinstance(other, CompleteTask):
            return other.task_uuid == self.task_uuid
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<task_uuid={self.task_uuid}>"


class ConfirmNeeded:

    def __init__(self, task_uuid: str, confirm_needed: bool, _type: str = ''):
        self._type = 'confirm_needed'
        self.task_uuid = task_uuid
        self.confirm_needed = confirm_needed

    def __eq__(self, other: object) -> bool:
        if isinstance(other, ConfirmNeeded):
            return other.task_uuid == self.task_uuid \
                   and other.confirm_needed == self.confirm_needed
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<task_uuid={self.task_uuid},confirm_needed={self.confirm_needed}>"




class Process:

    def __init__(self, uuid: str, name: str, tasks: [str], description: str = "", _type: str = ''):
        """
        :param name: name of the task
        :param tasks: list of task uuid's
        :param description: a description of the task
        :param uuid: UUID of the task
        """
        self._type = 'process'
        self.uuid = uuid
        self.name = name
        self.tasks: [str] = tasks
        self.description = description

        if len(tasks) <= 0:
            raise ValueError("A task must have a least 1 subtask or component!")

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Process):
            return other.uuid == self.uuid and \
                   other.name == self.name and \
                   other.tasks == self.tasks and \
                   other.description == self.description
        return False

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    def __repr__(self):
        return f"<id={self.uuid},name={self.name},tasks={self.tasks},description={self.description}>"
